# remove when we don't support py38 anymore
from __future__ import annotations
import argparse
import logging

from dotenv_cli.core import run_dotenv
from dotenv_cli import __VERSION__


logger = logging.getLogger(__name__)


def parse_args(args: list[str] | None = None) -> argparse.Namespace:
    """Parse arguments.

    Paramters
    ---------
    args
        This if for debugging only.

    Returns
    -------
    argparse.Namespace

    """
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-e',
        '--dotenv',
        help='Alternative .env file',
        default='.env',
    )

    parser.add_argument(
        'command',
        help='Shell command to execute',
        nargs=argparse.REMAINDER,
    )

    parser.add_argument(
        '--version',
        action='version',
        version=__VERSION__,
    )

    return parser.parse_args(args)


def main() -> int:
    """Run dotenv.

    This function parses sys.argv and runs dotenv.

    Returns
    -------
    int
        the return value

    """
    args = parse_args()
    if not args.command:
        return 0

    return run_dotenv(args.dotenv, args.command)



# remove when we don't support py38 anymore
from __future__ import annotations
import atexit
import logging
import os
from subprocess import Popen  # , PIPE, STDOUT


logger = logging.getLogger(__name__)


def read_dotenv(filename: str) -> dict[str, str]:
    """Read dotenv file.

    Parameters
    ----------
    filename
        path to the filename

    Returns
    -------
    dict

    """
    try:
        with open(filename, 'r') as fh:
            data = fh.read()
    except FileNotFoundError:
        logger.warning(f"{filename} does not exist, continuing without "
                       "setting environment variables.")
        data = ""

    res = {}
    for line in data.splitlines():
        logger.debug(line)

        line = line.strip()

        # ignore comments
        if line.startswith('#'):
            continue

        # ignore empty lines or lines w/o '='
        if '=' not in line:
            continue

        key, value = line.split('=', 1)

        # allow export
        if key.startswith('export '):
            key = key.split(' ', 1)[-1]

        key = key.strip()
        value = value.strip()

        # remove quotes (not sure if this is standard behaviour)
        if len(value) >= 2 and value[0] == value[-1] == '"':
            value = value[1:-1]
            # escape escape characters
            value = bytes(value, 'utf-8').decode('unicode-escape')

        elif len(value) >= 2 and value[0] == value[-1] == "'":
            value = value[1:-1]

        res[key] = value
    logger.debug(res)
    return res


def run_dotenv(filename: str, command: list[str]) -> int:
    """Run dotenv.

    This function executes the commands with the environment variables
    parsed from filename.

    Parameters
    ----------
    filename
        path to the .env file
    command
        command to execute

    Returns
    -------
    int
        the return value

    """
    # read dotenv
    dotenv = read_dotenv(filename)

    # update env
    env = os.environ.copy()
    env.update(dotenv)

    # execute
    proc = Popen(
        command,
        # stdin=PIPE,
        # stdout=PIPE,
        # stderr=STDOUT,
        universal_newlines=True,
        bufsize=0,
        shell=False,
        env=env,
    )

    def terminate_proc() -> None:
        """Kill child process.

        All signals should be forwarded to the child processes
        automatically, however child processes are also free to ignore
        some of them. With this we make sure the child processes get
        killed once dotenv exits.

        """
        proc.kill()

    # register
    atexit.register(terminate_proc)

    _, _ = proc.communicate()

    # unregister
    atexit.unregister(terminate_proc)

    return proc.returncode



import tempfile

import pytest

from dotenv_cli import core


def test_full() -> None:
    TEST = r"""
BASIC=basic basic
export EXPORT=foo
EMPTY=
INNER_QUOTES=this 'is' a test
INNER_QUOTES2=this "is" a test
TRIM_WHITESPACE= foo
KEEP_WHITESPACE="  foo  "
MULTILINE_DQ="multi\nline"
MULTILINE_SQ='multi\nline'
MULTILINE_NQ=multi\nline
# some comment
should be ignored
"""

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['BASIC'] == 'basic basic'
    assert env['EXPORT'] == 'foo'
    assert env['EMPTY'] == ''
    assert env['INNER_QUOTES'] == "this 'is' a test"
    assert env['INNER_QUOTES2'] == 'this "is" a test'
    assert env['TRIM_WHITESPACE'] == "foo"
    assert env['KEEP_WHITESPACE'] == "  foo  "
    assert env['MULTILINE_DQ'] == "multi\nline"
    assert env['MULTILINE_SQ'] == "multi\\nline"
    assert env['MULTILINE_NQ'] == "multi\\nline"

    assert len(env) == 10


def test_basic() -> None:
    """Basic unquoted strings"""
    TEST = "FOO=BAR"

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == 'BAR'


def test_empty() -> None:
    """Empty values become empty strings."""
    TEST = "FOO="

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == ''


def test_inner_quotes() -> None:
    """Inner quotes are mainained."""
    TEST = "\n".join(["FOO1=this 'is' a test",
                      'FOO2=this "is" a test'])

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO1'] == "this 'is' a test"
    assert env['FOO2'] == 'this "is" a test'


def test_trim_whitespaces() -> None:
    """Whitespaces are stripped from unquoted values."""
    TEST = "FOO=  test  "

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == "test"


def test_keep_whitespaces() -> None:
    """Whitespaces are mainteined from quoted values."""
    TEST = "FOO='  test  '"

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == "  test  "


def test_multiline() -> None:
    """Quoted values can contain newlines."""
    TEST = r'FOO="This is\nbar"'

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == 'This is\nbar'


@pytest.mark.parametrize('input_, expected', [
    ('FOO="Test"', 'Test'),
    ("FOO='Test'", 'Test'),
    ("FOO='\"Test\"'", '"Test"'),
    ('FOO="\'Test\'"', "'Test'"),
])
def test_quotes(input_: str, expected: str) -> None:
    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(input_)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == expected


def test_comments() -> None:
    """Lines starting with # are ignored."""
    TEST = """
    FOO=BAR
    # comment
    BAR=BAZ
    """

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert len(env) == 2
    assert env['FOO'] == 'BAR'
    assert env['BAR'] == 'BAZ'


def test_emtpy_lines() -> None:
    """Empty lines are skipped."""
    TEST = """
    FOO=BAR

    BAR=BAZ
    """

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert len(env) == 2
    assert env['FOO'] == 'BAR'
    assert env['BAR'] == 'BAZ'


def test_export() -> None:
    """Exports are allowed."""
    TEST = "export FOO=BAR"

    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write(TEST)

    env = core.read_dotenv(f.name)
    assert env['FOO'] == 'BAR'



from subprocess import run, PIPE
from pathlib import Path
import tempfile
from typing import Iterator

import pytest


DOTENV_FILE = """
# comment=foo
TEST=foo
TWOLINES='foo\nbar'
TEST_COMMENT=foo # bar
LINE_WITH_EQUAL='foo=bar'
"""


@pytest.fixture
def dotenvfile() -> Iterator[Path]:
    _file = Path.cwd() / '.env'
    with _file.open('w') as fh:
        fh.write(DOTENV_FILE)
    yield _file
    _file.unlink()


def test_stdout(dotenvfile: Path) -> None:
    proc = run(['dotenv', 'echo', 'test'], stdout=PIPE)
    assert b'test' in proc.stdout


def test_stderr(dotenvfile: Path) -> None:
    proc = run(['dotenv',
                'python3', '-c', 'import os; os.write(2, b"test")'],
               stderr=PIPE)
    assert b'test' in proc.stderr


def test_returncode(dotenvfile: Path) -> None:
    proc = run(['dotenv', 'false'])
    assert proc.returncode == 1

    proc = run(['dotenv', 'true'])
    assert proc.returncode == 0


def test_alternative_dotenv() -> None:
    with tempfile.NamedTemporaryFile('w', delete=False) as f:
        f.write('foo=bar')

    proc = run(['dotenv', '-e', f.name, 'env'], stdout=PIPE)
    assert b'foo=bar' in proc.stdout

    proc = run(['dotenv', '--dotenv', f.name, 'env'], stdout=PIPE)
    assert b'foo=bar' in proc.stdout


def test_nonexisting_dotenv() -> None:
    proc = run(['dotenv', '-e', '/tmp/i.dont.exist', 'true'], stderr=PIPE)
    assert proc.returncode == 0
    assert b'does not exist' in proc.stderr


def test_no_command() -> None:
    proc = run(['dotenv'])
    assert proc.returncode == 0



#!/usr/bin/env python


from setuptools import setup

meta = {}
exec(open('./dotenv_cli/version.py').read(), meta)
meta['long_description'] = open('./README.md').read()

setup(
    name='dotenv-cli',
    version=meta['__VERSION__'],
    description='Simple dotenv CLI.',
    long_description=meta['long_description'],
    long_description_content_type='text/markdown',
    keywords='dotenv cli .env',
    author='Bastian Venthur',
    author_email='mail@venthur.de',
    url='https://github.com/venthur/dotenv-cli',
    python_requires='>=3.7',
    extras_require={
      'dev': [
          'pytest',
          'pytest-cov',
          'flake8',
          'wheel',
          'twine',
      ]
    },
    packages=['dotenv_cli'],
    entry_points={
        'console_scripts': [
            'dotenv = dotenv_cli.cli:main'
        ]
    },
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)
